/****************************************************************************
 * NCSA HDF                                                                 *
 * Software Development Group                                               *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * COPYING file.                                                            *
 *                                                                          *
 ****************************************************************************/
 
/**************************************************************************

MAIN MODULES
  
   filetype          - file type?
   get_env           - Gets the environment variables that might be useful to us
   main              - process client request

****************************************************************************/

#ifdef RCSID
static char RcsId[] = "@(#)sdb.c,v 1.53 1996/05/17 19:42:23 xlu Exp";
#endif

/*#define DMAIN_C
#include "myutil.h"  */

/* Include our stuff */
#include "sdb.h"
#include "sdb_sds.h"
#include "sdb_pal.h"
#include "sdb_ri.h"
#include "sdb_ann.h"
#include "sdb_vg.h"
#include "sdb_vd.h"
#include "sdb_util.h"
#include "util.h"
#include "html_util.h"




/*
 * return value 0 -> not recognized
 *              1 -> HDF file
 *              2 -> netCDF file
 *             -1 -> FAIL
 */
int
filetype(char *fname)
{
    char hdf_type[4]    = {0x0e, 0x03, 0x13, 0x01};
    char netcdf_type[4] = {0x43, 0x44, 0x46, 0x01};
    char ftype[4] = {0,0,0,0};
    FILE *fhandle = NULL;
    int ret_value = 0;

    DBUG_PRINT(1,(LOGF,"filetype: opening %s\n",fname));
    if ((fhandle = fopen(fname,"r")) == NULL)
      {
          DBUG_PRINT(1,(LOGF,"filetype: failed to open file \n"));
          ret_value = -1;
          goto done;
      }
    if (fread(ftype,1,4,fhandle) == 0)
      {
          DBUG_PRINT(1,(LOGF,"filetype: failed to read 4 bytes \n"));
          ret_value = -1;
          goto done;
      }
    if (!memcmp(ftype,hdf_type,4))
      {
          DBUG_PRINT(1,(LOGF,"this is an HDF file\n"));
          ret_value = 1;
      }
    else if (!memcmp(ftype,netcdf_type,4))
      {
          DBUG_PRINT(1,(LOGF,"this is a netCDF file\n"));
          ret_value = 2;
      }
    else 
      {
          DBUG_PRINT(1,(LOGF,"this is an unknown file type\n"));
      }

    /* close file */
    fclose(fhandle);
  done:
    if (ret_value == -1)
      {
          if (fhandle != NULL)
              fclose(fhandle);
      }
    return ret_value;
} /* filetype() */

/*-------------------------------------------------------------------- 
 NAME
     get_env
 DESCRIPTION
      Gets the enviromental variables that might be useful to us      

 RETURNS
     
--------------------------------------------------------------------*/ 
void
get_env(hserv_st *s_env )
{
    /* Get relavant enironment variables passed to us by httpd server */
    s_env->request_method = getenv("REQUEST_METHOD");
    s_env->path_info      = getenv("PATH_INFO");
    DBUG_PRINT(1,(LOGF,"path_info=%s\n",s_env->path_info));
    s_env->path_translated = getenv("PATH_TRANSLATED");
    DBUG_PRINT(1,(LOGF,"path_translated=%s\n",s_env->path_translated));
    s_env->script_name     = getenv("SCRIPT_NAME");
    DBUG_PRINT(1,(LOGF,"script_name=%s\n",s_env->script_name));
    s_env->query_string    = getenv("QUERY_STRING");
    s_env->remote_host     = getenv("REMOTE_HOST");
    s_env->remote_addr     = getenv("REMOTE_ADDR");
    s_env->auth_type       = getenv("AUTH_TYPE");
    s_env->remote_user     = getenv("REMOTE_USER");
    s_env->content_type    = getenv("CONTENT_TYPE");
    s_env->content_length  = getenv("CONTENT_LENGTH");
    s_env->http_accept     = getenv("HTTP_ACCEPT");
    s_env->http_user_agent = getenv("HTTP_USER_AGENT");

} /* get_env() */

/* ----------------------------------------------------------------------

			      Main Function

   ---------------------------------------------------------------------- */
/*
 * do the main thang.
 *
 * Note that there is some UGLY code that got put in previously
 * to handle SDS which should be revamped in the next revision
 *
 */

#if defined(sun) || defined(__sun__) 
/* for SUN (Call Fortran Library in C ) */
int MAIN_;
#endif

int
main(int argc, char *argv[]) 
{
    entry entries[MAX_ENTRIES];  /* array to hold query string */
    register int x;              /* loop array */
    register int m=0;
    char *html_file;             /* HTML file name */
    int  target;              /* flag: Positive - HDF reference, NULL - HDF file */
    mime_content_type mime_type; /* reply type e.g. text/html */
    int no_path=FALSE;
    int current_entry=0;
    int errflag = 0;
    int no_query;
    char *cl;                    /* Pointer to query string */
    int  pcl;
    lvar_st  l_vars;            /* Local variables */
    hserv_st myserv_env;        /* Server environemnt variables */
    int file_type = -1;
  

#ifdef DEBUG
/*
   Parse config file, looking for location of debug log file.
   If the log file is not specified, or is incorrectly specified
   in the config file, send log messages to stderr, instead of exiting
   with an error
*/
 
char tmpword[MAX_STRING_LEN];
char l[MAX_STRING_LEN];
int n, k;
FILE *configfile;
int logfile_open=FALSE;



                      configfile=fopen(SDB_CONFIG_FILE, "r");
                      if(configfile){
                      while(!(cfg_getline(l,MAX_STRING_LEN,configfile))) {
                       ++n;
                       if((l[0] != '#') && (l[0] != '\0')) {
                           cfg_getword(tmpword,l);

                           if(!strcasecmp(tmpword, "Debug")) {
                           cfg_getword(tmpword,l);
                           DBUG_OPEN(tmpword,"w+");
                           logfile_open=TRUE;  
                           }
                           }
                           }
                           fclose(configfile);
                           if (logfile_open == FALSE) DBUG_OPEN("stderr","w+");
                      }

#endif /* DEBUG */

    DBUG_PRINT(1,(LOGF,"************SDB Gateway starting******** \n")); 

    /* Initialize local variables */
    do_init(&l_vars);

      { /* Processing a request from user*/

          DBUG_PRINT(1,(LOGF,"processing request from user \n"));

          /* get Server Environment variables */
          get_env(&myserv_env);
          DBUG_PRINT(1,(LOGF,"got environmnet variables \n"));
          l_vars.h_env = &myserv_env;

          /* switch on METHOD */
          if(!strcmp(myserv_env.request_method,"GET")) 
            {
                DBUG_PRINT(1,(LOGF,"METHOD is GET=%s\n",myserv_env.request_method));

                /* Get relative path */
                l_vars.hdf_path_t = myserv_env.path_translated;
                DBUG_PRINT(1,(LOGF,"hdf_path_t =%s \n", l_vars.hdf_path_t));

                current_entry = 1;

                /* Get the Query string */
                cl = getenv("QUERY_STRING");
                if (cl == NULL) 
                    gateway_err(stderr,"No HDF/netCDF file or reference number specified.\n",1);
                DBUG_PRINT(1,(LOGF,"query_string = %s \n", cl));

                /* Decode query string and place in "entries" array 
                 * may need to change since we use "hdfref;tag=x,ref=y" format
                 * which cause trouble with FORMs...*/
                for(x = 0; cl[0] != '\0'; x++) 
                  {
                      m = x;
                      entries[x].val = (char *)HDgetspace(128*sizeof(char));
                      entries[x].name = (char *)HDgetspace(128*sizeof(char));
                      getword(entries[x].val,cl,'&');
                      plustospace(entries[x].val);
                      unescape_url(entries[x].val);
                      getword(entries[x].name,entries[x].val,'|');
                      DBUG_PRINT(2,(LOGF,"entries[%d].name=%s \n", x,entries[x].name));
                      DBUG_PRINT(2,(LOGF,"entries[%d].val=%s \n", x,entries[x].val));
                  }

                /* Get HDF file name from entry */
                if (pull_filename(entries[0].name,&l_vars) == FAIL)
                    gateway_err(stderr,"error getting hdf filename \n",1);

                /* Get proper target ie. hdf ref or file name */
                target = pull_ref(entries[0].name,&l_vars);
                DBUG_PRINT(1,(LOGF,"target = %d \n", target));
                DBUG_PRINT(1,(LOGF,"myserv_env.path_info = %s \n", myserv_env.path_info));
                DBUG_PRINT(1,(LOGF,"l_vars.f_path_r = %s \n", l_vars.f_path_r));

                /* Check relative name path */
                if (strcmp(l_vars.f_path_r,"."))
                    l_vars.hdf_path = (char *)mk_path_name(2,'/',l_vars.f_path_r,l_vars.f_name);
                else
                    l_vars.hdf_path = strdup(l_vars.f_name);
                DBUG_PRINT(1,(LOGF,"hdf_path =%s \n", l_vars.hdf_path));

                DBUG_PRINT(1,(LOGF,"l_vars.hdf_path_t = %s \n", l_vars.hdf_path_t));
                /* Create full file name with path */
                if (strcmp(l_vars.hdf_path_t,"."))
                    l_vars.hdf_file = (char *)mk_path_name(2,'/',l_vars.hdf_path_t,l_vars.hdf_path);
                else
                    l_vars.hdf_file = strdup(l_vars.hdf_path);
                DBUG_PRINT(1,(LOGF,"hdf_file = %s \n", l_vars.hdf_file));

                /* determine the type of file */
                switch(file_type = filetype(l_vars.hdf_file))
                  {
                  case 1: /* HDF file */
                      l_vars.do_netcdf = 0;
                      break;
                  case 2: /* netCDF file */
                      l_vars.do_netcdf = 1;
                      break;
                  case -1: /* error */
                  case 0:
                  default:
                      gateway_err(stderr,"file may not exist or file format is not supported by DIB  ",1);  /* lpd */
                      break;
                  }

                if (file_type == 1 || file_type == 2)
                  {
                      /* Act upon whether it is a HDF reference number of file name */
                      if (target == FAIL) {                
                          l_vars.hdf_ref = entries[0].name;
                          if ((html_file=(char *)sdbGrokFile(l_vars.hdf_file, entries, 
                                                             &mime_type, current_entry, 
                                                             m,&l_vars)) == NULL)
                              gateway_err(stderr,"main: sdbGrokFile ",1);
                      }
                      else {
                          if (target == SDBPLANE) {
                              if ((html_file=(char*)sdsGrokFile(l_vars.hdf_file,entries, 
                                                                &mime_type, current_entry, 
                                                                m,&l_vars)) == NULL)
                                  gateway_err(stderr,"main: sdsGrokFile ",1);
                          }
	
                          else {
                              if (target == SDBREF) {
                                  if ((html_file=(char*)sdsGrokRef(l_vars.hdf_file,
                                                                   l_vars.hdf_ref, entries, 
                                                                   &mime_type, current_entry, 
                                                                   m,&l_vars)) == NULL)  
                                      gateway_err(stderr,"main: sdsGrokRef ",1);

                              }
                              else {
                                  if (target == SDBVIEW) {
	      
                                      if ((html_file = (char *)sdbGrokView(l_vars.hdf_file,
                                                                           l_vars.hdf_ref,entries, 
                                                                           &mime_type,current_entry,
                                                                           m,&l_vars))
                                          == NULL) 
                                          gateway_err(stderr,"main: sdbGrokView",1);
                                  }

                                  else {
              
                                      if ((html_file =(char *)sdbGrokRef(l_vars.hdf_file,
                                                                         l_vars.hdf_ref,
                                                                         entries,&mime_type, 
                                                                         current_entry, m,&l_vars))
                                          == NULL)
                                          gateway_err(stderr,"main: sdbGrokFile ",1);
                                  }
                              }
                          }
                      }
                  }
                
		/* send reply to client based on mime_type*/
                if (send_reply(stdout, html_file, mime_type,&l_vars) == FAIL)
                    gateway_err(stderr,"Error sending reply ",1);

                DBUG_PRINT(1,(LOGF,"Cleaning up... \n\n\n"));
                /* Clean up...*/
                if (l_vars.hdf_path != NULL)
                    FREE_CLEAR(l_vars.hdf_path);
                if (l_vars.hdf_file != NULL)
                    FREE_CLEAR(l_vars.hdf_file);
                if (l_vars.f_name != NULL)
                    FREE_CLEAR(l_vars.f_name);
                if (html_file != NULL)
                    free(html_file);
            }
          else if(!strcmp(myserv_env.request_method,"POST"))
            { /* We are using the POST method */
              
                DBUG_PRINT(1,(LOGF,"METHOD is POST=%s\n",myserv_env.request_method));
                if(strcmp(getenv("CONTENT_TYPE"),"application/x-www-form-urlencoded")) 
                  {
                      printf("This CGI script can only be used to decode form results. \n");
                      exit(1);
                  }
                pcl = atoi(getenv("CONTENT_LENGTH"));

                /* extract info off STDIN */
                for(x=0;pcl && (!feof(stdin));x++) 
                  {
                      m = x; /* number of entries */
                      entries[x].val = (char *)fmakeword(stdin,'&',(int *)&pcl);
                      plustospace(entries[x].val);
                      unescape_url(entries[x].val);
                      entries[x].name = (char *)makeword((char *)entries[x].val,'=');
                      DBUG_PRINT(2,(LOGF,"entries[%d].name=%s \n", x,entries[x].name));
                      DBUG_PRINT(2,(LOGF,"entries[%d].val=%s \n", x,entries[x].val));
                      /* ============ get variables ================ */
                      /* Get file name i.e. our target 
                       *  hdf_path is used for SDS's */
                      if(!strcmp(entries[x].name,"f_name") 
                         || !strcmp(entries[x].name,"hdf_path"))
                        {
                            if (pull_filename(entries[x].val,&l_vars) == FAIL)
                                gateway_err(stderr,"error getting hdf filename \n",1);
                        }

                      /* Get the new display mode */
                      if(!strcmp(entries[x].name,"display_mode"))
                          l_vars.display_mode = atoi(entries[x].val);

                      /* get tag/ref */
                      if(!strcmp(entries[x].name,"hdfref"))
                        {
                            l_vars.hdf_ref = entries[x].val;
                        }

                      /* Vdata field  names */
                      if(!strcmp(entries[x].name,"VH_FIELD"))
                        {
                            if (l_vars.field_indices == NULL)
                              {
                                  if ((l_vars.field_indices = (int *)HDmalloc(sizeof(int)*257)) == NULL)
                                      gateway_err(stderr,"error allocateing space for field indices \n",1);
                              }
                            l_vars.field_indices[l_vars.nfields] = atoi(entries[x].val);
                            l_vars.nfields++;
                        }
					  
                        /* parameters for x-y plotting -lpd- */
                       if(!strcmp(entries[x].name, "Plot_style")){
                          if(strcmp(entries[x].val, "Dot") == 0)
                                   l_vars.xyplot_style = -1;
                          else if (strcmp(entries[x].val, "Solid line")== 0)
                                   l_vars.xyplot_style = 0;
                          else if (strstr(entries[x].val, "Dot line_"))
                                   l_vars.xyplot_style = atoi(&(entries[x].val[9]));
                          else 
                                   l_vars.xyplot_style = -1;
                       }
                       else if(strcmp(entries[x].name, "scaling") == 0)
                              l_vars.scaling = atoi(entries[x].val);
                       
                      if(!strcmp(entries[x].name, "Plot_title"))
                             l_vars.xyplot_name = entries[x].val;
                      if(!strcmp(entries[x].name, "X_name"))
                             l_vars.x_axis_name = entries[x].val;
                      if(!strcmp(entries[x].name, "Y_name"))
                             l_vars.y_axis_name = entries[x].val;


                      /* Vdata field  names for Y axis in xy plotting-- lpd */
                      if(!strcmp(entries[x].name,"VH_YFIELD"))
                        {
                            if (l_vars.yfield_indices == NULL)
                              {
                                  if ((l_vars.yfield_indices = (int *)HDmalloc(sizeof(int)*257)) == NULL)
                                      gateway_err(stderr,"error allocateing space for y-axis field indices \n",1);
                              }
                            l_vars.yfield_indices[l_vars.nyfields] = atoi(entries[x].val);
                            l_vars.nyfields++;
                        }

                      /* get the total number of fields -- lpd */
                      if(!strcmp(entries[x].name, "NFIELDS"))
                            l_vars.ntfields = atoi(entries[x].val);

                      /* Get the Vdata start record */
                      if(!strcmp(entries[x].name,"VH_START"))
                          l_vars.vh_start_rec = atoi(entries[x].val);

                      /* Get the Vdata skip record -- lpd */
                      if(!strcmp(entries[x].name, "VH_SKIP"))
                          l_vars.vh_skip_rec = atoi(entries[x].val);

                      /* Get the Vdata ending record */
                      if(!strcmp(entries[x].name,"VH_END"))
                          l_vars.vh_end_rec = atoi(entries[x].val);

                      /* get the view method -- lpd */
                      if(!strcmp(entries[x].name, "retrieve type"))
                          l_vars.view_type = atoi(entries[x].val);

                      /* SDS rank dimensions */
                      if(!strcmp(entries[x].name,"SD_RANK"))
                        {
                            if (l_vars.sd_dim_start == NULL)
                              {
                                  if ((l_vars.sd_dim_start = (int32 *)HDmalloc(sizeof(int32)*(atoi(entries[x].val)))) == NULL)
                                      gateway_err(stderr,"error allocateing space for dim start  indices \n",1);
                              }
                            if (l_vars.sd_dim_stride == NULL)
                              {
                                  if ((l_vars.sd_dim_stride = (int32 *)HDmalloc(sizeof(int32)*(atoi(entries[x].val)))) == NULL)
                                      gateway_err(stderr,"error allocateing space for dim stride  indices \n",1);
                              }
                            if (l_vars.sd_dim_end == NULL)
                              {
                                  if ((l_vars.sd_dim_end = (int32 *)HDmalloc(sizeof(int32)*(atoi(entries[x].val)))) == NULL)
                                      gateway_err(stderr,"error allocateing space for dim end  indices \n",1);
                              }
                        }

                      /* SDS dimensions */
                      if(!strcmp(entries[x].name,"SD_DIM_START"))
                        {
                            if (l_vars.sd_dim_start != NULL)
                                l_vars.sd_dim_start[l_vars.ndims] = atoi(entries[x].val);
                        }

                      if(!strcmp(entries[x].name,"SD_DIM_STRIDE"))
                        {
                            if (l_vars.sd_dim_stride != NULL)
                                l_vars.sd_dim_stride[l_vars.ndims] = atoi(entries[x].val);
                        }

                      if(!strcmp(entries[x].name,"SD_DIM_END"))
                        {
                            if (l_vars.sd_dim_end != NULL)
                              {
                                  l_vars.sd_dim_end[l_vars.ndims] = atoi(entries[x].val);
                                  l_vars.ndims++; /* increment to next dimension */
                              }
                        }


                      /* ===variables for sds or phdu or image extension=== */
                      /* variables for IMAGE Extension */
                      if (!strcmp(entries[x].name,"SDB_PLANE"))
                          l_vars.sdb_plane = atoi(entries[x].val);
	    
                      if (!strcmp(entries[x].name,"SDB_IMGFLAG"))
                          l_vars.sdb_imgFlag = atoi(entries[x].val);

                      if (!strcmp(entries[x].name,"SDB_REF"))
                          l_vars.sdb_ref = atoi(entries[x].val);

                      if (!strcmp(entries[x].name,"SDB_START"))
                          l_vars.sdb_start_rec = atoi(entries[x].val);

                      if (!strcmp(entries[x].name,"SDB_END"))
                          l_vars.sdb_end_rec = atoi(entries[x].val);

                  } /* end for */

                /* Did we get a valid file name to work with */
                if(l_vars.f_name != NULL)
                  {
                      l_vars.hdf_path_t = myserv_env.path_translated;
                      /* Check relative name path */
                      if (strcmp(l_vars.f_path_r,"."))
                          l_vars.hdf_path = (char *)mk_path_name(2,'/',l_vars.f_path_r,l_vars.f_name);
                      else
                          l_vars.hdf_path = strdup(l_vars.f_name);
                      DBUG_PRINT(1,(LOGF,"hdf_path =%s \n", l_vars.hdf_path));

                      /* Create full file name with path */
                      if (l_vars.hdf_path_t != NULL && strcmp(l_vars.hdf_path_t,"."))
                          l_vars.hdf_file = (char *)mk_path_name(2,'/',l_vars.hdf_path_t,l_vars.hdf_path);
                      else
                          l_vars.hdf_file = strdup(l_vars.hdf_path);
                      DBUG_PRINT(1,(LOGF,"hdf_file = %s \n", l_vars.hdf_file));

                  }
                else
                    gateway_err(stderr,"error getting hdf filename \n",1);

		 
                /* determine the type of file */
                switch(file_type = filetype(l_vars.hdf_file))
                  {
                  case 1: /* HDF file */
                      l_vars.do_netcdf = 0;
                      break;
                  case 2: /* netCDF file */
                      l_vars.do_netcdf = 1;
                      break;
                  case -1: /* error */
                  case 0:
                  default:
                      gateway_err(stderr,"main: error in filetype ",0);
                      break;
                  }

                /* ========== make the variables(subsetting) valid ==========*/

                /* for sds or phdu or image extension */
                if (l_vars.sdb_imgFlag != 0) {
                    if (l_vars.sdb_plane <= 0)
                        l_vars.sdb_plane = 1;
                   if (l_vars.sdb_start_rec <= 0)
                        l_vars.sdb_start_rec = 1;
                    if (l_vars.sdb_end_rec <= 0)
                        l_vars.sdb_end_rec = l_vars.sdb_start_rec;
                    if ( l_vars.sdb_end_rec <l_vars.sdb_start_rec)
                        l_vars.sdb_start_rec= l_vars.sdb_end_rec;
                    if (l_vars.sdb_end_rec > l_vars.sdb_plane)
                        l_vars.sdb_end_rec = l_vars.sdb_plane;
                }

		 
                /* It is possible that a request of browsing 
                   one or MORE objects has been submitted, and 
                   so we need a loop to display all of them. 
                   - Start of UGLY code */
                for (x = 0; x <= m; x++) 
                  { 
                      char *tp1, *tp2;

                      DBUG_PRINT(1,(LOGF,"main: for x=%d \n", x));
		   
                      if (entries[x].val==NULL)
                          continue;

                      no_query = FALSE; /* Keep track of whether any requests have been 
                                           made. */

                      DBUG_PRINT(1,(LOGF,"sdsView(31)=%d\n",l_vars.sdb_imgFlag));

                      /* get the new Query_string */

                      /* ============ for sds ==========*/
                      if (l_vars.sdb_imgFlag == SDSVIEW) {
                          l_vars.hdf_ref = HDmalloc(100);
                          /* modify the reference nuber */
                          sprintf(l_vars.hdf_ref,"ref=%d,start=%d,end=%d",
                                  l_vars.sdb_ref,
                                  l_vars.sdb_start_rec,
                                  l_vars.sdb_end_rec);
                      }


                      /* An object has been selected from any of the 
                         scroll boxes, and so determine what the hell 
                         kind of object it is. */
                      if (l_vars.hdf_ref == NULL)
                        {
                            /*  HDF ref is either tag/ref pair or name of object */
                            if ((l_vars.hdf_ref = (char *)strrchr(entries[x].val, ';'))!=NULL)
                                ; /* tag ref pair */
                            else if ((l_vars.hdf_ref=(char *)strrchr(entries[x].val, '#'))!=NULL)
                                ; /* name of object */
                            else 
                              { /* pick first entry as name i.e. hdfref  */
                                  continue;
                              }
                        }

                      DBUG_PRINT(1,(LOGF,"l_vars.hdf_ref=%s\n",l_vars.hdf_ref)); 	  

                      /* damn */
                      if (l_vars.hdf_ref==NULL) 
                        {
                            fprintf(l_vars.hfp, "<B> Object not being handled. </B>\n");
                            if (send_reply(stdout, html_file, mime_type,&l_vars)==FAIL)
                                gateway_err(stderr,"Error sending reply.",1); 
                            else 
                                break;
                        }

                      /* For a scientific dataset, each request goes with a form 
                         containing three fields, namely, "start", "stride", and "end". 
                         So when a request of browsing a dataset is submitted via a 
                         form, it occupies four consecutive entries in "entries[x]", 
                         i.e. "name", "start", "stride", and "end". Obviously, we only 
                         want to process the first entry and the last three entries are
                         supplemental information. And the following block skips three 
                         iterations for the last three entries. */
                      if (x>0)
                        {
                            if (((tp1=(char *)strrchr(entries[x].val, '#'))==NULL)&&
                                ((tp2=(char *)strrchr(entries[x].val, ';'))==NULL))
                                continue;
                        }


                      /* are we dealing with a FITS file or HDF/netCDF */
                            if (l_vars.sdb_imgFlag == SDSVIEW) {
		  
                                html_file = (char *)sdbGrokView(l_vars.hdf_file, 
                                                                l_vars.hdf_ref, 
                                                                entries,
                                                                &mime_type, x, m,&l_vars);
                            }
                            else {			  
                                html_file = (char *)sdbGrokRef (l_vars.hdf_file, 
                                                                l_vars.hdf_ref, 
                                                                entries,
                                                                &mime_type, x, m,&l_vars);
                            }

                      if (html_file != NULL)
                          if (send_reply(stdout, html_file, mime_type,&l_vars)==FAIL)
                              gateway_err(stderr,"Error sending reply.",1);

                      if (html_file != NULL)
                          free(html_file);

                      DBUG_PRINT(1,(LOGF,"main: end for x=%d \n", x));
                      break;
                  } /* for (x...); processing all the requests. */
 
            }
          else /* neither METHOD of GET or POST */
              gateway_err(stderr,"This Gateway should be reference with a METHOD of GET or POST .\n",1);

      }

    /* cleanup */
    cleanup_sds(&l_vars);
    cleanup_pal(&l_vars);
    cleanup_rig(&l_vars);
    cleanup_an(&l_vars);
    cleanup_vgs(&l_vars);
    cleanup_vd(&l_vars);
    DBUG_CLOSE(LOGF);
    exit(0);
} /* end main() */


