/*  Author: Dr. Liping Di, Hughes STX Corporation

 Copyright (C) 1997 Hughes STX Corporation.
 Permission to use, modify, and distribute this software and its
 documentation for any purpose without fee is hereby granted, provided
 that the above copyright notice appear in all copies and that both that
 copyright notice and this permission notice appear in supporting
 documentation.

*/

#ifndef PLOT_H
#define PLOT_H

#define PLOT_DOT -1
#define PLOT_LINE_SOLID 0
#define PLOT_LINE_PATTERN_1 2
#define NO_FRAME 0
#define HAVE_FRAME 1

typedef struct xy_plot_struct {
       unsigned char *bit_map;       /* pointer to the bit map */
       unsigned short xsize;         /* bit map size in x direction= number of columns */
       unsigned short ysize;         /* bit map size in y direction= number of rows */
       unsigned char bvalue;         /* the background value to be set */

       unsigned short nvars;          /* number of plots, allow for muti-variable ploting */
       unsigned short *x, **y;        /* x, y pairs */
       char **var_names;              /* variable names */
       long *n_pairs;                 /* number of (x, y) pairs */
       short *plot_pattern;           /* -1 = dot, 0= soild line ... 6 */
       short *plot_value;             /* The forground value to be plot */

       unsigned short x_axis_start, x_axis_end;  /* start and end of x axis */
       unsigned short y_axis_start, y_axis_end;  /* start and end of y axis */
       unsigned char axis_value;       /* axis value (color) */
       short frame;                    /* 0= no frame, 1 = frame */ 
        
       unsigned short n_major_tic_x;   /* number of major tic in x direction */
       unsigned short tic_start_x;     /* start location of tic in x axis */
       unsigned short tic_step_x;      /* step of each major tic */

       unsigned short n_minor_tic_x;
       unsigned short mtic_start_x;
       unsigned short mtic_step_x;

       unsigned short n_major_tic_y;   /* number of major tic in y direction */
       unsigned short tic_start_y;     /* start location of tic in y axis */
       unsigned short tic_step_y;      /* step of each major tic */

       unsigned short n_minor_tic_y;
       unsigned short mtic_start_y;
       unsigned short mtic_step_y;

       /* second y tics */
       unsigned short n_1major_tic_y;   /* number of major tic in y direction */
       unsigned short tic_1start_y;     /* start location of tic in y axis */
       unsigned short tic_1step_y;      /* step of each major tic */

       unsigned short n_1minor_tic_y;
       unsigned short mtic_1start_y;
       unsigned short mtic_1step_y;

       short grid;                      /* grid =0, no grid, 1=grid */

       unsigned label_value;                 /* label value to be plot */

       unsigned short n_label_x;             /* number of lables for x  */
       unsigned short x_label_x, x_label_y;  /* xy coordinates of the start label */
       unsigned short x_label_step;
       char **label_x;                      /* the actual label for x */             

       unsigned short n_label_y;             /* number of lables for y  */
       unsigned short y_label_x, y_label_y;  /* xy coordinates of the start label */
       unsigned short y_label_step;
       char **label_y;                       /* the actual label for y */
    
       /* second y label */
       unsigned short n_1label_y;             /* number of lables for y  */
       unsigned short y_1label_x, y_1label_y;  /* xy coordinates of the start label */
       unsigned short y_1label_step;
       char **label_1y;                       /* the actual label for y */

       unsigned char name_value;             /* color for the name */
       char *x_axis_name;                    /* name of x axis */
       unsigned short xname_start_x, xname_start_y;

       char *y_axis_name;
       unsigned short yname_start_x, yname_start_y;

       /* second y_axis_name */
       char *y_1axis_name;
       unsigned short yname_1start_x, yname_1start_y;

       unsigned char title_value;
       char *title;
       unsigned short title_start_x, title_start_y;
       
} XY_PLOT;

extern int xy_plot(XY_PLOT *plot);
#endif
