/****************************************************************************
 * NCSA HDF                                                                 *
 * Software Development Group                                               *
 * National Center for Supercomputing Applications                          *
 * University of Illinois at Urbana-Champaign                               *
 * 605 E. Springfield, Champaign IL 61820                                   *
 *                                                                          *
 * For conditions of distribution and use, see the accompanying             *
 * COPYING file.                                                            *
 *                                                                          *
 ****************************************************************************/

/**************************************************************************
UTILITY MODULES
   InfoMsg - Print an informational message 
   gateway_err       - report gateway error to client(using stdout)
****************************************************************************/

/* Include our stuff */
#include "myutil.h"
#include "html_util.h"
#include "htmlstrings.h"
#include "glist.h"
#include "string.h"

/* ----------------------------------------------------------------------
 NAME
     InfoMsg - Print an informational message 
 DESCRIPTION
     Print an informational message   
 RETURNS
     NONE
-------------------------------------------------------------------------*/  
void InfoMsg(str)
char *str;
{
    printf("%s\n", str);
}

/*--------------------------------------------------------------------
 NAME
     print_strings  - prints an array of strings
 DESCRIPTION
     Given a array of strings, prints them out in order.
 RETURNS
     Nothing
--------------------------------------------------------------------*/
void
print_strings(FILE *output,
              char **str)
{
    int i;
    for(i=0; strlen(str[i]); i++)
      {
          fprintf(output,"%s\n", str[i]);
      }
} /* print_strings() */
 
/*-------------------------------------------------------------------- 
 NAME
     gateway_err - print error and exit
 DESCRIPTION
     Report that the gateway encountered an error.
 RETURNS
     Doesn't return but calls exit()
--------------------------------------------------------------------*/ 
void 
gateway_err(FILE *fout,  /* output stream */
            char *msg,   /* error message to output */
            int fatal)   /* 0= not fatal, 1= fatal error message */
{
    if (fatal)
      {

          /* If were are dummping just print the error */
                printf("Content-type: text/html%c%c",10,10);
                printf("<title>%s</title>",ERR3);
                printf("<h1>%s</h1>",ERR3);
                printf("%s:<p>",ERR4);
                printf("%s", msg);
	  exit(-1);   
      }
    else
        fprintf(fout,"<p> Error:<pre> %s <pre> \n",msg);

} /* gateway_err */


/*------------------------------------------------------------------------ 
 NAME
     print_label - Prints a single string to output file
 DESCRIPTION
     Prints a single string to output file
 RETURNS
     NONE
-------------------------------------------------------------------------*/  
void 
print_label(FILE *fout,  /* output stream */
            char *msg)   /* error message to output */
{
        fprintf(fout," %s",msg);

}


/*------------------------------------------------------------------------ 
 NAME
     prhtml_hyperlink - Prints a pathname as a html hyperlink 
 DESCRIPTION
     Prints a pathname as a html hyperlink 
 RETURNS
     NONE
-------------------------------------------------------------------------*/  

void 
prhtml_hyperlink(FILE *fout,  /* output stream */
            char *msg)   /* error message to output */
{
        fprintf(fout,"<A HREF=\"%s\"> %s </A><p>",msg, msg);

}



/*------------------------------------------------------------------------ 
 NAME
     prhtml_boldlabel - Prints a message as a bold html label
 DESCRIPTION
     Prints a message as a bold html label
 RETURNS
     NONE
-------------------------------------------------------------------------*/  

void 
prhtml_boldlabel(FILE *fout,  /* output stream */
            char *msg)   /* error message to output */
{
        fprintf(fout,"<B> %s</B><P>",msg);

}

/*------------------------------------------------------------------------ 
 NAME
     prhtml_title - Prints a HTML title in a variety of formats.
 DESCRIPTION
    Prints a HTML title in a variety of formats.
 RETURNS
     NONE
 INPUTS
     1) pointer to output file 2) title
     3) number of objects to be displayed
     4) label that will be displayed for one object presented
     5) and 6) are two labels that will be displayed if more than one 
     object is displayed "There are 2 palettes in this file", for example
     7) Optional additional phrase. If none, input NULL

-------------------------------------------------------------------------*/  

void 
prhtml_title(FILE *fout,  /* output stream */
            char *title, int count, char *label_for_one, 
            char *label_for_many1, char *label_for_many2,
            char *optional_phrase)  
{
    if(title != NULL)
    {
        print_label(fout, "<HR>\n");
        fprintf(fout, "<H2>%s</H2>\n",title);
    }

    if(count == 1)
    { 
        if(label_for_one != NULL) 
           fprintf(fout, "%s\n", label_for_one);
    }
    else
    {
        if((label_for_many1 != NULL) && (label_for_many2 != NULL)) 
           fprintf(fout, "%s%d%s\n", label_for_many1, count, label_for_many2);
    } 
    if(optional_phrase != NULL)
        fprintf(fout, " %s\n", optional_phrase);

}


/*------------------------------------------------------------------------ 
 NAME
     prhtml_3str - prints 3 strings to output file
 DESCRIPTION
     Prints three strings 
 RETURNS
     NONE
-------------------------------------------------------------------------*/  
void 
prhtml_3str(FILE *fout, char *str1, char *str2, char *str3 )
{
        fprintf(fout,"%s %s %s", str1,str2,str3);
}


/*------------------------------------------------------------------------ 
 NAME
     prhtml_H2label - Prints a message as a size 2 HTML title
 DESCRIPTION
     Prints a message as a size 2 HTML title
 RETURNS
     NONE
-------------------------------------------------------------------------*/  
void 
prhtml_H2label(FILE *fout,  /* output stream */
            char *msg)  
{
        fprintf(fout,"<H2> %s</H2>\n",msg);
}




/*------------------------------------------------------------------------ 
 NAME
     prhtml_submitbutton - Prints a HTML submit button 
 DESCRIPTION
     Prints a HTML submit button in a variery of common forms
     Optionally prints a reset button, the hdf tag and ref info, 
     file name, if input value is not NULL for these fields 
 RETURNS
     NONE
 INPUTS
 1) pointer to output file,   2) character string before submit button
 3) label on submit button    4)label on reset button 
 5) hdf tag to send as a hidden variable   6) hdf ref to send as a hidden variable
 7) hdf subsample number  8) string of HDF file name
 9) display_mode to send as hidden variable  

-------------------------------------------------------------------------*/  


void 
prhtml_submitbutton(FILE *fout, char *title, char *submitmsg, char *resetmsg, int tag, int ref, int subnum, char *name, int display_mode)
{
          fprintf(fout,"<br>%s", title);
          fprintf(fout, "<INPUT TYPE=\"submit\" VALUE=\"%s\"> \n",submitmsg);
          if (resetmsg != NULL)
          	fprintf(fout, "<INPUT TYPE=\"reset\" VALUE=\"%s\"> <P>\n",resetmsg);
          if (tag != 0)  /* valid tag is 1-65535 */
          	fprintf(fout, "<INPUT TYPE=\"hidden\" NAME=\"hdfref\" VALUE=\";tag=%d,ref=%d,s=%d\"> ",
                  tag, ref, subnum);
          if (name != NULL)
          	{
		fprintf(fout, "<INPUT TYPE=\"hidden\" NAME=\"f_name\" VALUE=\"%s\">\n",
                  name);
          	fprintf(fout, "<INPUT TYPE=\"hidden\" NAME=\"display_mode\" VALUE=\"%d\">\n",
                  display_mode);
		}


} 


/*------------------------------------------------------------------------ 
 NAME
     prhtml_select_outputtype - Prints a radio button group in HTML form
 DESCRIPTION
     Prints a radio button group in HTML form for user to select mode
     for outputting data.
 RETURNS
     NONE
 INPUTS
     1)pointer to output file,    2)string of HDF file name
     3) HDF tag     4) HDF ref number
     5) HDF sub_sample number    6) title printed before radio buttons
     7) array of strings, representing radio buttons to be printed
     8) num of radio button strings in above array
  NOTES
     The first string in the input array is the name of the string in the rest
    of the list that is designated as the "selected" radio button  

-------------------------------------------------------------------------*/  

void 
prhtml_select_outputtype(FILE *fout, char *title, char *button_array[], int numbuttons)
{
    int i=1;

       fprintf(fout, "<p>\n<hr><PRE WIDTH = \"132\" >%s: <p>",title);

       for (i = 1; i < numbuttons; i++)
       { 

          /* is this radio button the designated "selected" one ?, are the strings the same? */
          if ( strcmp(button_array[i], button_array[0]) ==0 )  
                fprintf(fout, "<INPUT type=\"radio\" NAME=\"retrieve type\" VALUE=%d checked>%s",i,button_array[i]);
          else
		fprintf(fout, "<INPUT type=\"radio\" NAME=\"retrieve type\" VALUE=%d>%s",i,button_array[i]);
         }


          fprintf(fout, "\n<p></pre><hr>"); 

} 


/*----------------------------------------------------------------------- 
 NAME
       add_tblhdr_str - Adds a table header label to the list of table headers.
 DESCRIPTION
       Adds a table header label to the list of table headers.
       Should only be called by HTML Printing Table Functions.
 RETURNS
      SUCCEED/FAIL
-----------------------------------------------------------------------*/
int
add_tblhdr_str(Generic_list *tbl_list, /* tb list to add this hdr to */
       char *hdr)             /* header itself */
{
    tbl_st *tbl_elem  = NULL;
    int    ret_value = 1;     /* SUCCESS */

    /* allocate space for annotation element */
    if ((tbl_elem = (tbl_st *)malloc(sizeof(tbl_st))) == NULL)
      {
          gateway_err(stderr,"add_: failed to allocate space for tbl_elem \n",0);
          ret_value = 0; 
          goto done;
      }   

    /* fill in the appropriate info */
    tbl_elem->hdr = hdr;

    /* add to the list */
    add_to_end(*tbl_list, tbl_elem); 
    DBUG_PRINT(3,(LOGF,"add_: adding tblhdr, hdr=%s \n", tbl_elem->hdr)); 

  done:
    return ret_value;
} /* add_tblhdr_str */



/*----------------------------------------------------------------------- 
 NAME
       prhtml_hdrentry - Prints a HTML table header label
 DESCRIPTION
       Prints a HTML table header label from the list of table headers.
       Should only be called by HTML Printing Table Functions.
 RETURNS
      SUCCEED/FAIL
-----------------------------------------------------------------------*/

void
prhtml_hdrentry(void *dptr, /* hdr string */
          void *args)
{
    tbl_st *list = dptr; 
    FILE *fout = args;

	if (list != NULL) 
        {
	   if (list->hdr != NULL) 
           {
              prhtml_3str(fout,"<TH>", list->hdr,"</TH>");
              DBUG_PRINT(3,(LOGF,"prhtml_hdrentry:  hdr=%s \n", list->hdr));} 
	} 
} /* prhtml_hdrentry() */


/*----------------------------------------------------------------------- 
 NAME
       free_tbl - Frees a table element
 DESCRIPTION
       Frees a table element.
       Should only be called by HTML Printing Table Functions.
 RETURNS
      SUCCEED/FAIL
 NOTES
-----------------------------------------------------------------------*/
void
free_tbl(void *dptr, void *args)             /* table element to free */
         
{
    tbl_st *tbl_elem = dptr;

    if (tbl_elem != NULL)
          free(tbl_elem); /* free element */

} /* free_tbl() */
 

									
/*----------------------------------------------------------------------- 
 NAME
       prhtml_tblhdr - Prints a HTML table
 DESCRIPTION
       Prints a HTML table , given an array of table headers
 RETURNS
      SUCCEED/FAIL
 INPUTS
      1) pointer to file to output data to
      2) string of caption
      3) array of table header labels
      4) number of entries in array of table headers

 NOTES
     this procedure still needs to be improved to
     1) free the Generic list it mallocs
     2) merge in table caption and title into hdr array or build new construct
     to do this
     3) INPUT AND PRINT TABLE DATA

-----------------------------------------------------------------------*/

void 
prhtml_tblhdr(FILE *fout, char *caption, char *hdrarray[], int numofhdrs)
{
	Generic_list *tbl_list;
        int    ret_value = 1;     /* SUCCESS */
        int currenthdrnum=0;

        /* allocate list to hold table headers */ 
          if ((tbl_list = (Generic_list *)malloc(sizeof(Generic_list))) == NULL)
            {
                gateway_err(fout,"prhtml_tb: failed to allocate space for tbl list\n",0);
                return;
            }


	/* initialize list */
          initialize_list(tbl_list);

        for ( currenthdrnum = 0; currenthdrnum < numofhdrs; ++currenthdrnum)
        {
        add_tblhdr_str(tbl_list, hdrarray[currenthdrnum]);
        }  

         
        if (caption != NULL)
        	prhtml_3str(fout, "<caption align=\"top\">", caption,"</caption>");

	/* Print Table Header Labels */
       
	print_label(fout,"<TR>");
        perform_on_list(*tbl_list,prhtml_hdrentry,fout); 
	print_label(fout,"</TR>");
         
        /* free tbl_list */
 
        reset_to_beginning(*tbl_list);
        if (tbl_list != NULL)
        {
          perform_on_list(*tbl_list,free_tbl,NULL);
          destroy_list(tbl_list);
          free(tbl_list);
          tbl_list = NULL;
         }

}
/*----------------------------------------------------------------------- 
 NAME
       prhtml_wholetbl - Prints an entire HTML given that each tbl entry
       contains the same type of data.
 DESCRIPTION
       Prints a whole HTML table , given a list of data and
       a pointer to a function THAT IS TO BE APPLIED
       TO EVERY DATA TYPE IN THE LIST
 RETURNS
      SUCCEED/FAIL
 INPUTS
      1) pointer to file to output data to
      2) Generic list of data
      3) function pointer of function to performed on 2), Generic list of data
-----------------------------------------------------------------------*/



void 
prhtml_wholetbl(FILE *fout, Generic_list list, void (*fn)(void *pointer, void *args))
{

        if (fn != NULL)
	{
         	perform_on_list(list,fn,fout);  
	}

}

/*----------------------------------------------------------------------- 
 NAME
       prhtml_tblentry - Prints a HTML table entry
 DESCRIPTION
       Prints a HTML table entry , given a list of data and
       an array of function pointers, one for each column

 RETURNS
      SUCCEED/FAIL
 INPUTS
      1) pointer to file to output data to
      2) Miscellaneous arguments to be given to function.  The lvars data
         structure is usually sent so other data about the item can be
         printed if desired.
      3) Generic list of data
      4) function pointer of function to performed on 3), Generic list of data
-----------------------------------------------------------------------*/

void 
prhtml_tblentry(FILE *fout, void *miscarg, void *arglist, void (*fn)(void *pointer, void *args))
{
        int    ret_value = 1;     /* SUCCESS */


	/* Print Table Entry */
        
        if (fn != NULL)
	{
		print_label(fout,"<TD>"); 
                (*fn)(arglist,miscarg);   
		print_label(fout,"</TD>");  
	}

}


/*------------------------------------------------------------------------ 
 NAME
     prhtml_form - Prints a HTML "select box" form
 DESCRIPTION
     Given a Generic list of name, build and print a HTML "select box" form.
     Also prints title for select box, and inputs other data values 
     such as display_mode and HDF file name, as "hidden values" to be
     transmitted via the form.
 RETURNS
     NONE
 INPUTS
     1) file pointer for output
     2) form title
     3) name of script form will invoke
     4) pathinfo value
     5) SELECT NAME = thisvalue
     6) SIZE = this value
     7) Phrase before submit button "To select a ..., press this buton"
     8) label on Submit button "Select X"
     9) Name of HDF file
     10)display_mode value
     11) Generic list of items to containing items in which to build the Select form
     12) pointer to function that will operate on Generic list in 11)
     13) Number of items in Generic list 11)
     14) Label used for presentation of items in Generic list 11)

-------------------------------------------------------------------------*/  


void 
prhtml_form(FILE *fout, char *formtitle, char *scriptname, char *pathinfo, char *selectlabel, int size, char *buttonlabel, char *browsebutton, char *browsefilename, int display_mode,Generic_list list, void (*fn)(Generic_list list, FILE *fout, int numinlist, char *lbl ), int numoflistitems, char *itemlabel)
{
 
        if (formtitle != NULL)
           fprintf(fout,"<p>%s",formtitle);
         
        fprintf(fout,"<FORM METHOD=\"POST\" ");
        fprintf(fout, "ACTION=\"%s%s\">\n",scriptname, pathinfo);
        fprintf(fout, "<SELECT NAME=\"%s\" SIZE=%d>\n",selectlabel, size);
    DBUG_PRINT(3,(LOGF,"prhtml_form selectlabel=%s,size=%d \n",selectlabel,size)); 
 
       
	/* Print Form Entries */
        
       if (fn != NULL)
       (*fn)(list,fout,numoflistitems,itemlabel);   

	/* Print out submit button for form */

         print_label(fout,"</SELECT>");
         prhtml_submitbutton(fout,buttonlabel,browsebutton,NULL,0,NULL,0,browsefilename,display_mode);
         print_label(fout,"</FORM>");

} 

/*----------------------------------------------------------------------- 
 NAME
       prhtml_tblselectentry - Prints a HTML table entry in a scrollable selectbox
 DESCRIPTION
       Prints a HTML table entry as a scrollable selectbox , given a list of data and
       an array of function pointers, one for each column

 RETURNS
      SUCCEED/FAIL
 INPUTS
      1) pointer to file to output data to
      2) Generic list of data
      3) function pointer of function to performed on 2), Generic list of data
-----------------------------------------------------------------------*/

void 
prhtml_tblselectentry(FILE *fout, void *miscarg, void *arglist, void (*fn)(void *pointer, void *args))
{
 
	print_label(fout,"<TD>"); 
        fprintf(fout,"<FORM METHOD=\"POST\" "); 
        fprintf(fout, "ACTION=\" \">\n"); 
        fprintf(fout, "<SELECT NAME=\" \" SIZE=4>\n" );
       
	/* Print Form Entries */
        
       if (fn != NULL)
       (*fn)(arglist,miscarg);   

         print_label(fout,"</SELECT>");
        print_label(fout,"</FORM>"); 

	print_label(fout,"</TD>");  
} 


